% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/S3Print.R
\name{print}
\alias{print}
\alias{print.CDM}
\alias{print.validation}
\alias{print.sim.data}
\alias{print.summary.CDM}
\alias{print.summary.validation}
\alias{print.summary.sim.data}
\title{Print Methods for Various Objects}
\usage{
\method{print}{CDM}(x, ...)

\method{print}{validation}(x, ...)

\method{print}{sim.data}(x, ...)

\method{print}{summary.CDM}(x, ...)

\method{print}{summary.validation}(x, ...)

\method{print}{summary.sim.data}(x, ...)
}
\arguments{
\item{x}{An object of the appropriate class (e.g., \code{\link[Qval]{CDM}}, 
\code{\link[Qval]{validation}}, \code{\link[Qval]{sim.data}}, or their summaries).}

\item{...}{Currently unused.  Additional arguments are ignored.}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print concise, user-friendly summaries of objects generated by the Qval package.
Supports objects of classes \code{\link[Qval]{CDM}}, \code{\link[Qval]{validation}}, \code{\link[Qval]{sim.data}},
as well as their corresponding summary objects.
}
\details{
The \code{print} methods provide an at-a-glance view of key information:
\describe{
  \item{print.CDM}{displays sample size, item and attribute counts, and package information.}
  \item{print.validation}{shows suggested modifications to the Q-matrix, marking changed entries with an asterisk.}
  \item{print.sim.data}{reports dimensions of simulated data and offers guidance on extraction.}
  \item{print.summary.CDM}{prints fitted model details and alpha-pattern distribution from a \code{summary.CDM} object.}
  \item{print.summary.validation}{prints suggested Q-matrix changes or a message if none are recommended.}
  \item{print.summary.sim.data}{prints attribute-pattern frequencies and proportions from \code{summary.sim.data}.}
}
}
\section{Methods (by class)}{
\itemize{
\item \code{print(CDM)}: Print method for CDM objects

\item \code{print(validation)}: Print method for validation objects

\item \code{print(sim.data)}: Print method for sim.data objects

\item \code{print(summary.CDM)}: Print method for summary.CDM objects

\item \code{print(summary.validation)}: Print method for summary.validation objects

\item \code{print(summary.sim.data)}: Print method for summary.sim.data objects

}}
\examples{
set.seed(123)
library(Qval)

\donttest{
################################################################
# Example 1: print a CDM object                                #
################################################################
Q <- sim.Q(3, 20)
IQ <- list(P0 = runif(20, 0.0, 0.2), P1 = runif(20, 0.8, 1.0))
data.obj <- sim.data(Q = Q, N = 500, IQ = IQ, 
                     model = "GDINA", distribute = "horder")
CDM.obj <- CDM(data.obj$dat, Q, model = "GDINA", 
               method = "EM", maxitr = 2000, verbose = 1)
print(CDM.obj)


################################################################
# Example 2: print a validation object                         #
################################################################
set.seed(123)
MQ <- sim.MQ(Q, 0.1)
CDM.obj <- CDM(data.obj$dat, MQ)
validation.obj <- validation(data.obj$dat, MQ, CDM.obj, 
                             method = "GDI")
print(validation.obj)


################################################################
# Example 3: print a sim.data object                           #
################################################################
set.seed(123)
Q2 <- sim.Q(3, 10)
data.obj2 <- sim.data(Q = Q2, N = 1000)
print(data.obj2)


################################################################
# Example 4: print summary objects                             #
################################################################
summary.CDM.obj <- summary(CDM.obj)
print(summary.CDM.obj)

summary.val.obj <- summary(validation.obj)
print(summary.val.obj)

summary.sim.obj <- summary(data.obj2)
print(summary.sim.obj)
}

}
